import os
import platform
import re
import subprocess

from setuptools.msvc import winreg
from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.core.constants import DEFAULT_USER_HOME_CACHE_PATH
import shutil
import requests
import xml.etree.ElementTree as elemTree
import functools

from webdriver_manager.core.os_manager import OSType, ChromeType


def get_web_chrome_link(version):
    """get selenium chromedriver version by major version got from local chrome"""
    print(f"Try to get target chrome driver in web: {version}")

    res = requests.get("https://chromedriver.storage.googleapis.com")
    root = elemTree.fromstring(res.content)
    for k in root.iter('{http://doc.s3.amazonaws.com/2006-03-01}Key'):
        if k.text.find(version + '.') == 0:
            ver = k.text.split('/')[0]
            print(f"found web_version: {ver}")
            return ver


def get_browser_version_from_os(browser_name):
    if platform.system() == 'Windows':
        # Use Windows-specific command to get browser version
        cmd = f'reg query "HKEY_LOCAL_MACHINE\\Software\\{browser_name}\\CurrentVersion" /v Version'
        output = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT, text=True)
        version_match = re.search(r'Version\s+REG_SZ\s+([\d.]+)', output)
        if version_match:
            return version_match.group(1)
    elif platform.system() == 'Darwin':
        # Use macOS-specific command to get browser version (for Safari)
        if browser_name.lower() == 'safari':
            cmd = '/usr/bin/safari --version'
            output = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT, text=True)
            version_match = re.search(r'version\s+([\d.]+)', output)
            if version_match:
                return version_match.group(1)
    elif platform.system() == 'Linux':
        # You may need to adapt this part for specific Linux distributions
        pass

    return None  # Return None if the version couldn't be determined
def get_driver_path():
    # WDM call a Window system cmd that be blocked, so use an custom validator collector instead
    _os_name = OSType.WIN if os.name == 'nt' else OSType.LINUX
    v = _custom_get_win_chrome_version() if _os_name == OSType.WIN else None
    if not v:
        v = get_browser_version_from_os(ChromeType.GOOGLE)

    print(f"found chromedriver version: {v}")

    version = v.split(".")[0] if v else None

    if _os_name == OSType.WIN:
        file_name = f"chromedriver_{version}.exe"
    else:
        file_name = f"chromedriver_{version}"

    expect_path = os.path.join(DEFAULT_USER_HOME_CACHE_PATH, file_name)

    if not os.path.exists(expect_path):
        wdm_path = ChromeDriverManager(version=get_web_chrome_link(version), cache_valid_range=1000).install()
        shutil.copy(wdm_path, expect_path)

    return expect_path


class DriverPath:

    @functools.cached_property
    def path(self):
        return get_driver_path()


driver_path = DriverPath()


def _get_chrome_installed(hive, flag):
    """get installed chrome version from winreg"""
    try:
        import winreg

        a_reg = winreg.ConnectRegistry(None, hive)
        a_key = winreg.OpenKey(a_reg, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall", 0, winreg.KEY_READ | flag)
        a_sub = winreg.OpenKey(a_key, "Google Chrome")
        return winreg.QueryValueEx(a_sub, "DisplayVersion")[0]
    except:
        pass


def _custom_get_win_chrome_version():
    """get chromedriver version, download if needed"""
    try:
        if os.path.exists("C:\\Program Files (x86)\\Google\\Chrome\\Application\\chrome.exe"):
            output = subprocess.check_output(
                r'wmic datafile where name="C:\\Program Files (x86)\\Google\\Chrome\\Application\\chrome.exe" '
                r'get Version /value', shell=True)
        else:
            output = subprocess.check_output(
                r'wmic datafile where name="C:\\Program Files\\Google\\Chrome\\Application\\chrome.exe" '
                r'get Version /value', shell=True)

        version = output.decode('utf-8').strip().split("=")[1]

        return version

    except:
        for hive, flag in ((winreg.HKEY_LOCAL_MACHINE, winreg.KEY_WOW64_32KEY),
                           (winreg.HKEY_LOCAL_MACHINE, winreg.KEY_WOW64_64KEY),
                           (winreg.HKEY_CURRENT_USER, 0)):
            cr_ver = _get_chrome_installed(hive, flag)
            if cr_ver is not None:
                return cr_ver
